"use strict";
/*
 * Copyright (c) 2014-2022 Bjoern Kimminich & the OWASP Juice Shop contributors.
 * SPDX-License-Identifier: MIT
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.solveFixIt = exports.solveFindIt = exports.thaw = exports.toSimpleIpAddress = exports.parseJsonCustom = exports.determineDisabledEnv = exports.disableOnWindowsEnv = exports.disableOnContainerEnv = exports.randomHexString = exports.jwtFrom = exports.downloadToFile = exports.extractFilename = exports.toISO8601 = exports.toMMMYY = exports.findChallengeById = exports.findChallengeByName = exports.notSolved = exports.sendNotification = exports.solve = exports.solveIf = exports.ctfFlag = exports.version = exports.trunc = exports.unquote = exports.containsOrEscaped = exports.containsEscaped = exports.contains = exports.endsWith = exports.startsWith = exports.isUrl = exports.queryResultToJson = void 0;
/* jslint node: true */
const package_json_1 = __importDefault(require("../package.json"));
const sequelize_1 = require("sequelize");
const fs = require("fs");
const colors = require('colors/safe');
const notifications = require('../data/datacache').notifications;
const challenges = require('../data/datacache').challenges;
const sanitizeHtml = require('sanitize-html');
const jsSHA = require('jssha');
const Entities = require('html-entities').AllHtmlEntities;
const config = require('config');
const entities = new Entities();
const download = require('download');
const crypto = require('crypto');
const clarinet = require('clarinet');
const isDocker = require('is-docker');
const isHeroku = require('is-heroku');
const isWindows = require('is-windows');
const logger = require('./logger');
const webhook = require('./webhook');
const antiCheat = require('./antiCheat');
const accuracy = require('./accuracy');
const models = require('../models');
const months = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC'];
let ctfKey;
if (process.env.CTF_KEY !== undefined && process.env.CTF_KEY !== '') {
    ctfKey = process.env.CTF_KEY;
}
else {
    fs.readFile('ctf.key', 'utf8', (err, data) => {
        if (err != null) {
            throw err;
        }
        ctfKey = data;
    });
}
const queryResultToJson = (data, status) => {
    var _a;
    let wrappedData = {};
    if (data) {
        if (!data.length && data.dataValues) {
            wrappedData = data.dataValues;
        }
        else if (data.length > 0) {
            wrappedData = [];
            for (let i = 0; i < data.length; i++) {
                wrappedData.push(((_a = data[i]) === null || _a === void 0 ? void 0 : _a.dataValues) ? data[i].dataValues : data[i]);
            }
        }
        else {
            wrappedData = data;
        }
    }
    return {
        status: status || 'success',
        data: wrappedData
    };
};
exports.queryResultToJson = queryResultToJson;
const isUrl = (url) => {
    return (0, exports.startsWith)(url, 'http');
};
exports.isUrl = isUrl;
const startsWith = (str, prefix) => str ? str.indexOf(prefix) === 0 : false;
exports.startsWith = startsWith;
const endsWith = (str, suffix) => str ? str.includes(suffix, str.length - suffix.length) : false;
exports.endsWith = endsWith;
const contains = (str, element) => str ? str.includes(element) : false; // TODO Inline all usages as this function is not adding any functionality to String.includes
exports.contains = contains;
const containsEscaped = function (str, element) {
    return (0, exports.contains)(str, element.replace(/"/g, '\\"'));
};
exports.containsEscaped = containsEscaped;
const containsOrEscaped = function (str, element) {
    return (0, exports.contains)(str, element) || (0, exports.containsEscaped)(str, element);
};
exports.containsOrEscaped = containsOrEscaped;
const unquote = function (str) {
    if (str && (0, exports.startsWith)(str, '"') && (0, exports.endsWith)(str, '"')) {
        return str.substring(1, str.length - 1);
    }
    else {
        return str;
    }
};
exports.unquote = unquote;
const trunc = function (str, length) {
    str = str.replace(/(\r\n|\n|\r)/gm, '');
    return (str.length > length) ? str.substr(0, length - 1) + '...' : str;
};
exports.trunc = trunc;
const version = (module) => {
    if (module) {
        return package_json_1.default.dependencies[module];
    }
    else {
        return package_json_1.default.version;
    }
};
exports.version = version;
const ctfFlag = (text) => {
    const shaObj = new jsSHA('SHA-1', 'TEXT'); // eslint-disable-line new-cap
    shaObj.setHMACKey(ctfKey, 'TEXT');
    shaObj.update(text);
    return shaObj.getHMAC('HEX');
};
exports.ctfFlag = ctfFlag;
const solveIf = function (challenge, criteria, isRestore = false) {
    if ((0, exports.notSolved)(challenge) && criteria()) {
        (0, exports.solve)(challenge, isRestore);
    }
};
exports.solveIf = solveIf;
const solve = function (challenge, isRestore = false) {
    challenge.solved = true;
    challenge.save().then((solvedChallenge) => {
        logger.info(`${isRestore ? colors.grey('Restored') : colors.green('Solved')} ${solvedChallenge.difficulty}-star ${colors.cyan(solvedChallenge.key)} (${solvedChallenge.name})`);
        (0, exports.sendNotification)(solvedChallenge, isRestore);
        if (!isRestore) {
            const cheatScore = antiCheat.calculateCheatScore(challenge);
            if (process.env.SOLUTIONS_WEBHOOK) {
                webhook.notify(solvedChallenge, cheatScore).catch((error) => {
                    logger.error('Webhook notification failed: ' + colors.red(error.message));
                });
            }
        }
    });
};
exports.solve = solve;
const sendNotification = function (challenge, isRestore) {
    if (!(0, exports.notSolved)(challenge)) {
        const flag = (0, exports.ctfFlag)(challenge.name);
        const notification = {
            key: challenge.key,
            name: challenge.name,
            challenge: challenge.name + ' (' + entities.decode(sanitizeHtml(challenge.description, { allowedTags: [], allowedAttributes: [] })) + ')',
            flag: flag,
            hidden: !config.get('challenges.showSolvedNotifications'),
            isRestore: isRestore
        };
        const wasPreviouslyShown = notifications.find(({ key }) => key === challenge.key) !== undefined;
        notifications.push(notification);
        if (global.io && (isRestore || !wasPreviouslyShown)) {
            global.io.emit('challenge solved', notification);
        }
    }
};
exports.sendNotification = sendNotification;
const notSolved = (challenge) => challenge && !challenge.solved;
exports.notSolved = notSolved;
const findChallengeByName = (challengeName) => {
    for (const c in challenges) {
        if (Object.prototype.hasOwnProperty.call(challenges, c)) {
            if (challenges[c].name === challengeName) {
                return challenges[c];
            }
        }
    }
    logger.warn('Missing challenge with name: ' + challengeName);
};
exports.findChallengeByName = findChallengeByName;
const findChallengeById = (challengeId) => {
    for (const c in challenges) {
        if (Object.prototype.hasOwnProperty.call(challenges, c)) {
            if (challenges[c].id === challengeId) {
                return challenges[c];
            }
        }
    }
    logger.warn('Missing challenge with id: ' + challengeId);
};
exports.findChallengeById = findChallengeById;
const toMMMYY = (date) => {
    const month = date.getMonth();
    const year = date.getFullYear();
    return months[month] + year.toString().substring(2, 4);
};
exports.toMMMYY = toMMMYY;
const toISO8601 = (date) => {
    let day = '' + date.getDate();
    let month = '' + (date.getMonth() + 1);
    const year = date.getFullYear();
    if (month.length < 2)
        month = '0' + month;
    if (day.length < 2)
        day = '0' + day;
    return [year, month, day].join('-');
};
exports.toISO8601 = toISO8601;
const extractFilename = (url) => {
    let file = decodeURIComponent(url.substring(url.lastIndexOf('/') + 1));
    if (this.contains(file, '?')) {
        file = file.substring(0, file.indexOf('?'));
    }
    return file;
};
exports.extractFilename = extractFilename;
const downloadToFile = async (url, dest) => {
    return download(url).then((data) => {
        fs.writeFileSync(dest, data);
    }).catch((err) => {
        logger.warn('Failed to download ' + url + ' (' + err.statusMessage + ')');
    });
};
exports.downloadToFile = downloadToFile;
const jwtFrom = ({ headers }) => {
    if (headers === null || headers === void 0 ? void 0 : headers.authorization) {
        const parts = headers.authorization.split(' ');
        if (parts.length === 2) {
            const scheme = parts[0];
            const token = parts[1];
            if (/^Bearer$/i.test(scheme)) {
                return token;
            }
        }
    }
    return undefined;
};
exports.jwtFrom = jwtFrom;
const randomHexString = (length) => {
    return crypto.randomBytes(Math.ceil(length / 2)).toString('hex').slice(0, length);
};
exports.randomHexString = randomHexString;
const disableOnContainerEnv = () => {
    return (isDocker() || isHeroku) && !config.get('challenges.safetyOverride');
};
exports.disableOnContainerEnv = disableOnContainerEnv;
const disableOnWindowsEnv = () => {
    return isWindows();
};
exports.disableOnWindowsEnv = disableOnWindowsEnv;
const determineDisabledEnv = (disabledEnv) => {
    if (isDocker()) {
        return disabledEnv && (disabledEnv === 'Docker' || disabledEnv.includes('Docker')) ? 'Docker' : null;
    }
    else if (isHeroku) {
        return disabledEnv && (disabledEnv === 'Heroku' || disabledEnv.includes('Heroku')) ? 'Heroku' : null;
    }
    else if (isWindows()) {
        return disabledEnv && (disabledEnv === 'Windows' || disabledEnv.includes('Windows')) ? 'Windows' : null;
    }
    return null;
};
exports.determineDisabledEnv = determineDisabledEnv;
const parseJsonCustom = (jsonString) => {
    const parser = clarinet.parser();
    const result = [];
    parser.onkey = parser.onopenobject = k => {
        result.push({ key: k, value: null });
    };
    parser.onvalue = v => {
        result[result.length - 1].value = v;
    };
    parser.write(jsonString).close();
    return result;
};
exports.parseJsonCustom = parseJsonCustom;
const toSimpleIpAddress = (ipv6) => {
    if ((0, exports.startsWith)(ipv6, '::ffff:')) {
        return ipv6.substr(7);
    }
    else if (ipv6 === '::1') {
        return '127.0.0.1';
    }
    else {
        return ipv6;
    }
};
exports.toSimpleIpAddress = toSimpleIpAddress;
const thaw = (frozenObject) => {
    return JSON.parse(JSON.stringify(frozenObject));
};
exports.thaw = thaw;
const solveFindIt = async function (key, isRestore) {
    const solvedChallenge = challenges[key];
    await models.Challenge.update({ codingChallengeStatus: 1 }, { where: { key, codingChallengeStatus: { [sequelize_1.Op.lt]: 2 } } });
    logger.info(`${isRestore ? colors.grey('Restored') : colors.green('Solved')} 'Find It' phase of coding challenge ${colors.cyan(solvedChallenge.key)} (${solvedChallenge.name})`);
    if (!isRestore) {
        accuracy.storeFindItVerdict(solvedChallenge.key, true);
        accuracy.calculateFindItAccuracy(solvedChallenge.key);
        antiCheat.calculateFindItCheatScore(solvedChallenge);
    }
};
exports.solveFindIt = solveFindIt;
const solveFixIt = async function (key, isRestore) {
    const solvedChallenge = challenges[key];
    await models.Challenge.update({ codingChallengeStatus: 2 }, { where: { key } });
    logger.info(`${isRestore ? colors.grey('Restored') : colors.green('Solved')} 'Fix It' phase of coding challenge ${colors.cyan(solvedChallenge.key)} (${solvedChallenge.name})`);
    if (!isRestore) {
        accuracy.storeFixItVerdict(solvedChallenge.key, true);
        accuracy.calculateFixItAccuracy(solvedChallenge.key);
        antiCheat.calculateFixItCheatScore(solvedChallenge);
    }
};
exports.solveFixIt = solveFixIt;
//# sourceMappingURL=utils.js.map